
using System;

namespace ODE.Integrators
{
	
	/// <summary>
	/// Calculates and returns the dydx derivatives.
	/// Do not modify the y array.
	/// </summary>
	public delegate void DerivativesDelegate(double x, double[] y, out double[] dydx);
	
	/// <summary>
	/// Defines the direction in which the stopping criteria is active.
	/// Positive means the stopping criteria is only met when approached from below.
	/// Negative means it's only met if approached from above.
	/// Both means the stopping criteria is met in either direction.
	/// </summary>
	public enum StoppingCriteriaDirection { Both, Positive, Negative };
	
	/// <summary>
	/// The roots (zeros) of this function is used as a stopping criteria for the integrator.
	/// The direction specifies in which direction the root must be approached in.
	/// </summary>
	public delegate double StoppingCriteriaDelegate(double x, double[] y, 
	                                                out StoppingCriteriaDirection direction);
	
	/// <summary>
	/// Abstract base class for explicit numerical integration of ODEs
	/// </summary>
	public interface IIntegrator
	{
		/// <summary>
		/// Sets the initial state of the integrator
		/// </summary>
		/// <param name="x0">
		/// Initial independent variable (time, position or similar)
		/// </param>
		/// <param name="y0">
		/// Initial dependent variable(s) (velocity, acceleration or similar)
		/// </param>
		/// <param name="h">
		/// Integration step length. May be modified by adapative integrators.
		/// </param>
		void SetInitialState(double x0, double[] y0, double h);
			
		/// <summary>
		/// Retrieves the state of the solver. 
		/// </summary>
		/// <param name="x">
		/// Independent variable
		/// </param>
		/// <param name="y">
		/// Dependend variable(s)
		/// </param>
		void GetState(out double x, out double[] y);

		/// <summary>
		/// Updates the state of the solver by performing an integration step.
		/// </summary>
		/// <param name="derivatives">
		/// A <see cref="DerivativesDelegate"/> used to retrive the derivatives for a given state.
		/// </param>
		void Update(DerivativesDelegate derivatives);
		
		/// <summary>
		/// Updates the state of the solver by performing an integration step.
		/// Halts the integration if the stopping criteria is met, 
		/// that is when the stoppingCriteria delegate returns zero.
		/// If this happens the stopped flag is set to true.
		/// </summary>
		/// <param name="derivatives">
		/// A <see cref="DerivativesDelegate"/> used to retrive the derivatives for a given state.
		/// </param>
		/// <param name="stoppingCriteria">
		/// A <see cref="StoppingCriteriaDelegate"/> used to get the stopping criteria.
		/// </param>
		/// <param name="stopped">
		/// Set to true if the stopping criteria was met, false otherwise.
		/// </param>
		void Update(DerivativesDelegate derivatives, StoppingCriteriaDelegate stoppingCriteria,
		            out bool stopped);
	}
}
