
using System;

namespace ODE.Integrators
{
		
	public abstract class IntegratorBase : IIntegrator
	{
		
		protected IntegratorBase()
		{
		}
		
		public virtual void SetInitialState(double x0, double[] y0, double h) {
			this.x = x0;
			this.n = y0.Length;
			this.y = new double[n];
			for (int i = 0; i < n; i++)
				this.y[i] = y0[i];
			this.h = h;
		}
		
		public void GetState(out double x, out double[] y) {
			x = this.x;
			y = new double[n];
			for (int i = 0; i < n; i++)
				y[i] = this.y[i];
		}
		
		public abstract void Update(DerivativesDelegate derivatives);
		
		public void Update(DerivativesDelegate derivatives, 
		                   StoppingCriteriaDelegate stoppingCriteria, out bool stopped)
		{
			double x_old;
			double[] y_old;
			GetState(out x_old, out y_old);
			
			StoppingCriteriaDirection direction;
			double sc_old = stoppingCriteria(x, y, out direction);
			
			stopped = false;
			
			int iter = 0;
			while (iter < 100)
			{
				// store step length in case of adaptive integrators
				double h_old = h;
				Update(derivatives);
				
				double sc = stoppingCriteria(x, y, out direction);
				
				// not crossed zero, everything ok
				if (sc * sc_old >= 0)
				{
					stopped = sc == 0;
					return;
				}
				
				// want positive but crossed in negative direction
				if (direction == StoppingCriteriaDirection.Positive && sc < 0)
					return;

				// want negative but crossed in positive direction
				if (direction == StoppingCriteriaDirection.Negative && sc > 0)
					return;
				
				// reduce step length and restart
				iter++;
				x = x_old;
				y_old.CopyTo(y, 0);
				
				if (h <= 1e-9)
					break;
				
				h = Math.Max(-h_old*sc_old / (sc - sc_old), 1e-9);
			}
			Console.WriteLine("Iter: {0}", iter);
			stopped = true;
		}
		
		// internal state array
		protected double[] y;
		protected int n;
		protected double x;
		protected double h;
	}
}
